
#include "lowreceiver.h"

LowReceiver::LowReceiver(short port_low, 
		short port_high, 
		Group &grp, 
		Clock &clk) : 
	_port_low(port_low), _port_high(port_high), _group(grp), _clock(clk) 
{

	printf("LowReceiver::LowReceiver --\n");

	_socket_desc = socket(AF_INET, SOCK_DGRAM, 0);

	/* et l'autre variante : AF_UNIX */
	if (_socket_desc < 0){
		/* error */
		perror("Creation de la socket impossible\n");
		fprintf(stderr,"BOUM at %s:%d\n",__FILE__,__LINE__);
		// FIXME: throw something
		exit(-1);

	}

	_socket_addr = new sockaddr_in;


	// port_low = externe
	// port_high = interne
	bzero(_socket_addr,sizeof(sockaddr_in));
	_socket_addr->sin_family = AF_INET;
	_socket_addr->sin_port = htons(_port_low);
	_socket_addr->sin_addr.s_addr = htonl(INADDR_ANY);

	// chopper une socket
	if (bind(_socket_desc,
				(struct sockaddr *)_socket_addr,
				sizeof(sockaddr_in)) < 0) {
		//FIXME : throw something
		perror("Attachement de la socket impossible\n");
		fprintf(stderr,"BOUM at %s:%d\n",__FILE__,__LINE__);
		exit(-1);
	}
}

LowReceiver::~LowReceiver() {
	delete _socket_addr;
}

void LowReceiver::run(){
	int buffer_len = 1500;
	char * buffer = new char[buffer_len];
	struct sockaddr_in repaddr;
	socklen_t sockaddr_len = sizeof(struct sockaddr_in);
	Message * mesg = NULL;
	while(1){
		// recevoir les donnes
		memset(buffer, '\0', buffer_len);
		bzero(&repaddr,sizeof(struct sockaddr_in));
		int read_buffer_len = recvfrom(
				_socket_desc,
				buffer,
				buffer_len,
				0,
				(struct sockaddr*)&repaddr,
				&sockaddr_len);

		mesg = new Message(buffer, read_buffer_len);
		printf("LowReceiver::run -- READ size %d\n", read_buffer_len);
		char * str = new char[mesg->getDataSize() + 1];
		strncpy(str, mesg->getData(), mesg->getDataSize());
		str[mesg->getDataSize()] = '\0';
		printf("LowReceiver::run -- READ '%s'\n", str);

		this->manage(mesg);

		delete(mesg);
		mesg = NULL;
		// on dispatche les donnes en fonction du protocole...
	}
}


void LowReceiver::manage(Message * mesg){
	switch(mesg->getType()){
		case Protocol::TYPE_TEST :
			{
				printf("LowReceiver::manage -- NOT IMPLEMENTED\n");
			}
			break;
		case Protocol::TYPE_ABCAST :
			{
				this->manage_abcast(mesg);
			}
			break;
		case Protocol::TYPE_CBCAST :
			{
				this->manage_cbcast(mesg);
			}
			break;
		default:
			printf("LowReceiver::manage -- ERROR\n");
			break;
	}
}

